from .preprocessing import load_image
from .models import load_model


class OCR:
    def __init__(self, model_name='microsoft/Florence-2-base-ft'):
        self.processor , self.model = load_model(model_name)

    def recognize_text(self, image_input, bbox=True):
        from PIL import Image

        prompt = '<OCR_WITH_REGION>' 

        if isinstance(image_input, str):
            image = load_image(image_input)
        elif isinstance(image_input, Image.Image):
            image = image_input
        else:
            raise ValueError("image_input must be a file path or a PIL image")

        inputs = self.processor(text=prompt, images=image, return_tensors="pt")
        generated_ids = self.model.generate(
        input_ids=inputs["input_ids"],
        pixel_values=inputs["pixel_values"],
        max_new_tokens=1024,
        early_stopping=False,
        do_sample=False,
        num_beams=3,
        )

        generated_text = self.processor.batch_decode(generated_ids, skip_special_tokens=False)[0]
        parsed_answer = self.processor.post_process_generation(
            generated_text, 
            task=prompt, 
            image_size=(image.width, image.height)
        )
     
        if not bbox:
            parsed_answer = " ".join(parsed_answer[prompt]["labels"])
        return parsed_answer





from transformers import AutoProcessor, AutoModelForCausalLM  
from unittest.mock import patch
from .preprocessing import fixed_get_imports
from PIL import Image

def load_model(model_name='microsoft/Florence-2-base-ft'):
    processor = AutoProcessor.from_pretrained(model_name, trust_remote_code=True)
    with patch("transformers.dynamic_module_utils.get_imports", fixed_get_imports): #workaround for unnecessary flash_attn requirement
        model = AutoModelForCausalLM.from_pretrained(model_name, trust_remote_code=True).eval()
    return processor, model




import unittest
from dulu import OCR
from PIL import Image
import requests

class TestOCR(unittest.TestCase):
    def setUp(self):
        self.ocr = OCR()
        self.image_url =  "https://github.com/whiletruelearn/dulu/blob/main/tests/sample_image.png?raw=true"
        self.image = Image.open(requests.get(self.image_url, stream=True).raw).convert('RGB')

    def test_recognize_text_without_bbox(self):
        text = self.ocr.recognize_text(self.image, bbox=False)
        self.assertIsInstance(text, str)
        self.assertIn("Gareth James", text)

    def test_recognize_text_with_bbox(self):
        result = self.ocr.recognize_text(self.image)
        self.assertIsInstance(result, dict)
        self.assertIn('<OCR_WITH_REGION>', result)
        self.assertIn('labels', result['<OCR_WITH_REGION>'])
        self.assertIn('Gareth James', result['<OCR_WITH_REGION>']['labels'][0])

if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

setup(
    name='dulu',
    version='0.1.3',
    packages=find_packages(), 
    install_requires=[
        'numpy',
        'opencv-python',
        'transformers',
        'torch',
        'timm',
        'einops'
    ],
    author='Krishna Sangeeth',
    author_email='kskrishnasangeeth@gmail.com',
    description='A Python library for OCR',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/whiletruelearn/florence_ocr',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)



from PIL import Image
from transformers.dynamic_module_utils import get_imports

def load_image(image_path):
    image = Image.open(image_path).convert('RGB')
    return image


def fixed_get_imports(filename="modeling_florence2.py") -> list[str]:
    if not str(filename).endswith("modeling_florence2.py"):
        return get_imports(filename)
    imports = get_imports(filename)
    imports.remove("flash_attn")
    return imports